@@ -8,14 +8,16 @@ Copyright (c) 2014 Arve Knudsen <arve.knudsen@gmail.com>
 BSD License
 """
 
-__author__ = ('Sam McCormack',
-              'Gerard Marull-Paretas <gerard@teslabs.com>, '
-              'Mark Harviston <mark.harviston@gmail.com>, '
-              'Arve Knudsen <arve.knudsen@gmail.com>')
-__version__ = '0.9.5'
-__url__ = 'https://github.com/CabbageDevelopment/qasync'
-__license__ = 'BSD'
-__all__ = ['QEventLoop', 'QThreadExecutor', 'asyncSlot', 'asyncClose']
+__author__ = (
+    "Sam McCormack",
+    "Gerard Marull-Paretas <gerard@teslabs.com>, "
+    "Mark Harviston <mark.harviston@gmail.com>, "
+    "Arve Knudsen <arve.knudsen@gmail.com>",
+)
+__version__ = "0.13.0"
+__url__ = "https://github.com/CabbageDevelopment/qasync"
+__license__ = "BSD"
+__all__ = ["QEventLoop", "QThreadExecutor", "asyncSlot", "asyncClose"]
 
 import asyncio
 import functools
@@ -33,21 +35,38 @@ logger = logging.getLogger(__name__)
 QtModule = None
 
 # If QT_API env variable is given, use that or fail trying
-QtModuleName = os.getenv('QT_API', '').strip()
-if QtModuleName:
-    logger.info('Forcing use of {} as Qt Implementation'.format(QtModuleName))
+qtapi_env = os.getenv("QT_API", "").strip().lower()
+if qtapi_env:
+    env_to_mod_map = {
+        "pyqt5": "PyQt5",
+        "pyqt": "PyQt4",
+        "pyqt4": "PyQt4",
+        "pyside6": "PySide6",
+        "pyside2": "PySide2",
+        "pyside": "PySide",
+    }
+    if qtapi_env in env_to_mod_map:
+        QtModuleName = env_to_mod_map[qtapi_env]
+    else:
+        raise ImportError(
+            "QT_API environment variable set ({}) but not one of [{}].".format(
+                qtapi_env, ", ".join(env_to_mod_map.keys())
+            )
+        )
+
+    logger.info("Forcing use of {} as Qt Implementation".format(QtModuleName))
     QtModule = importlib.import_module(QtModuleName)
 
 # If a Qt lib is already imported, use that
 if not QtModule:
-    for QtModuleName in ('PyQt5', 'PySide2'):
+    for QtModuleName in ("PyQt5", "PySide2", "PySide6"):
         if QtModuleName in sys.modules:
             QtModule = sys.modules[QtModuleName]
             break
 
 # Try importing qt libs
 if not QtModule:
-    for QtModuleName in ('PyQt5', 'PySide2'):
+    for QtModuleName in ("PyQt5", "PySide2", "PySide6"):
         try:
             QtModule = importlib.import_module(QtModuleName)
         except ImportError:
@@ -56,23 +75,31 @@ if not QtModule:
             break
 
 if not QtModule:
-    raise ImportError('No Qt implementations found')
+    raise ImportError("No Qt implementations found")
 
-logger.info('Using Qt Implementation: {}'.format(QtModuleName))
+logger.info("Using Qt Implementation: {}".format(QtModuleName))
 
-QtCore = importlib.import_module(QtModuleName + '.QtCore', package=QtModuleName)
-QtGui = importlib.import_module(QtModuleName + '.QtGui', package=QtModuleName)
-if QtModuleName == 'PyQt5':
+QtCore = importlib.import_module(QtModuleName + ".QtCore", package=QtModuleName)
+QtGui = importlib.import_module(QtModuleName + ".QtGui", package=QtModuleName)
+
+if QtModuleName == "PyQt5":
     from PyQt5 import QtWidgets
     from PyQt5.QtCore import pyqtSlot as Slot
 
     QApplication = QtWidgets.QApplication
-elif QtModuleName == 'PySide2':
+
+elif QtModuleName == "PySide2":
     from PySide2 import QtWidgets
     from PySide2.QtCore import Slot
 
     QApplication = QtWidgets.QApplication
 
+elif QtModuleName == "PySide6":
+    from PySide6 import QtWidgets
+    from PySide6.QtCore import Slot
+
+    QApplication = QtWidgets.QApplication
+
 from ._common import with_logger  # noqa
 
 
@@ -84,11 +111,13 @@ class _QThreadWorker(QtCore.QThread):
     For use by the QThreadExecutor
     """
 
-    def __init__(self, queue, num):
+    def __init__(self, queue, num, stackSize=None):
         self.__queue = queue
         self.__stop = False
         self.__num = num
         super().__init__()
+        if stackSize is not None:
+            self.setStackSize(stackSize)
 
     def run(self):
         queue = self.__queue
@@ -100,26 +129,27 @@ class _QThreadWorker(QtCore.QThread):
 
             future, callback, args, kwargs = command
             self._logger.debug(
-                '#{} got callback {} with args {} and kwargs {} from queue'
-                    .format(self.__num, callback, args, kwargs),
+                "#{} got callback {} with args {} and kwargs {} from queue".format(
+                    self.__num, callback, args, kwargs
+                ),
             )
             if future.set_running_or_notify_cancel():
-                self._logger.debug('Invoking callback')
+                self._logger.debug("Invoking callback")
                 try:
                     r = callback(*args, **kwargs)
                 except Exception as err:
-                    self._logger.debug('Setting Future exception: {}'.format(err))
+                    self._logger.debug("Setting Future exception: {}".format(err))
                     future.set_exception(err)
                 else:
-                    self._logger.debug('Setting Future result: {}'.format(r))
+                    self._logger.debug("Setting Future result: {}".format(r))
                     future.set_result(r)
             else:
-                self._logger.debug('Future was canceled')
+                self._logger.debug("Future was canceled")
 
-        self._logger.debug('Thread #{} stopped'.format(self.__num))
+        self._logger.debug("Thread #{} stopped".format(self.__num))
 
     def wait(self):
-        self._logger.debug('Waiting for thread #{} to stop...'.format(self.__num))
+        self._logger.debug("Waiting for thread #{} to stop...".format(self.__num))
         super().wait()
 
 
@@ -137,11 +167,21 @@ class QThreadExecutor:
     ...     assert r == 4
     """
 
-    def __init__(self, max_workers=10):
+    def __init__(self, max_workers=10, stack_size=None):
         super().__init__()
         self.__max_workers = max_workers
         self.__queue = Queue()
-        self.__workers = [_QThreadWorker(self.__queue, i + 1) for i in range(max_workers)]
+        if stack_size is None:
+            # Match cpython/Python/thread_pthread.h
+            if sys.platform.startswith("darwin"):
+                stack_size = 16 * 2 ** 20
+            elif sys.platform.startswith("freebsd"):
+                stack_size = 4 * 2 ** 20
+            elif sys.platform.startswith("aix"):
+                stack_size = 2 * 2 ** 20
+        self.__workers = [
+            _QThreadWorker(self.__queue, i + 1, stack_size) for i in range(max_workers)
+        ]
         self.__been_shutdown = False
 
         for w in self.__workers:
@@ -153,8 +193,10 @@ class QThreadExecutor:
 
         future = Future()
         self._logger.debug(
-            'Submitting callback {} with args {} and kwargs {} to thread worker queue'
-                .format(callback, args, kwargs))
+            "Submitting callback {} with args {} and kwargs {} to thread worker queue".format(
+                callback, args, kwargs
+            )
+        )
         self.__queue.put((future, callback, args, kwargs))
         return future
 
@@ -167,7 +209,7 @@ class QThreadExecutor:
 
         self.__been_shutdown = True
 
-        self._logger.debug('Shutting down')
+        self._logger.debug("Shutting down")
         for i in range(len(self.__workers)):
             # Signal workers to stop
             self.__queue.put(None)
@@ -257,9 +299,9 @@ class _QEventLoop:
     ...     loop.run_until_complete(xplusy(2, 2))
     """
 
-    def __init__(self, app=None, set_running_loop=True):
+    def __init__(self, app=None, set_running_loop=True, already_running=False):
         self.__app = app or QApplication.instance()
-        assert self.__app is not None, 'No QApplication has been instantiated'
+        assert self.__app is not None, "No QApplication has been instantiated"
         self.__is_running = False
         self.__debug_enabled = False
         self.__default_executor = None
@@ -278,15 +320,20 @@ class _QEventLoop:
         if set_running_loop:
             asyncio.events._set_running_loop(self)
 
+        # We have to set __is_running to True after calling
+        # super().__init__() because of a bug in BaseEventLoop.
+        if already_running:
+            self.__is_running = True
+
     def run_forever(self):
         """Run eventloop forever."""
         self.__is_running = True
         self._before_run_forever()
 
         try:
-            self._logger.debug('Starting Qt event loop')
+            self._logger.debug("Starting Qt event loop")
             rslt = self.__app.exec_()
-            self._logger.debug('Qt event loop ended with result {}'.format(rslt))
+            self._logger.debug("Qt event loop ended with result {}".format(rslt))
             return rslt
         finally:
             self._after_run_forever()
@@ -294,7 +341,7 @@ class _QEventLoop:
 
     def run_until_complete(self, future):
         """Run until Future is complete."""
-        self._logger.debug('Running {} until complete'.format(future))
+        self._logger.debug("Running {} until complete".format(future))
         future = asyncio.ensure_future(future, loop=self)
 
         def stop(*args):
@@ -307,21 +354,21 @@ class _QEventLoop:
             future.remove_done_callback(stop)
         self.__app.processEvents()  # run loop one last time to process all the events
         if not future.done():
-            raise RuntimeError('Event loop stopped before Future completed.')
+            raise RuntimeError("Event loop stopped before Future completed.")
 
-        self._logger.debug('Future {} finished running'.format(future))
+        self._logger.debug("Future {} finished running".format(future))
         return future.result()
 
     def stop(self):
         """Stop event loop."""
         if not self.__is_running:
-            self._logger.debug('Already stopped')
+            self._logger.debug("Already stopped")
             return
 
-        self._logger.debug('Stopping event loop...')
+        self._logger.debug("Stopping event loop...")
         self.__is_running = False
         self.__app.exit()
-        self._logger.debug('Stopped event loop')
+        self._logger.debug("Stopped event loop")
 
     def is_running(self):
         """Return True if the event loop is running, False otherwise."""
@@ -338,7 +385,7 @@ class _QEventLoop:
         if self.is_closed():
             return
 
-        self._logger.debug('Closing event loop...')
+        self._logger.debug("Closing event loop...")
         if self.__default_executor is not None:
             self.__default_executor.shutdown()
 
@@ -347,7 +394,9 @@ class _QEventLoop:
         self._timer.stop()
         self.__app = None
 
-        for notifier in itertools.chain(self._read_notifiers.values(), self._write_notifiers.values()):
+        for notifier in itertools.chain(
+            self._read_notifiers.values(), self._write_notifiers.values()
+        ):
             notifier.setEnabled(False)
 
         self._read_notifiers = None
@@ -358,14 +407,20 @@ class _QEventLoop:
         if asyncio.iscoroutinefunction(callback):
             raise TypeError("coroutines cannot be used with call_later")
         if not callable(callback):
-            raise TypeError('callback must be callable: {}'.format(type(callback).__name__))
+            raise TypeError(
+                "callback must be callable: {}".format(type(callback).__name__)
+            )
 
         self._logger.debug(
-            'Registering callback {} to be invoked with arguments {} after {} second(s)'
-                .format(callback, args, delay))
+            "Registering callback {} to be invoked with arguments {} after {} second(s)".format(
+                callback, args, delay
+            )
+        )
 
         if sys.version_info >= (3, 7):
-            return self._add_callback(asyncio.Handle(callback, args, self, context=context), delay)
+            return self._add_callback(
+                asyncio.Handle(callback, args, self, context=context), delay
+            )
         return self._add_callback(asyncio.Handle(callback, args, self), delay)
 
     def _add_callback(self, handle, delay=0):
@@ -394,15 +449,16 @@ class _QEventLoop:
         else:
             # this is necessary to avoid race condition-like issues
             existing.setEnabled(False)
-            existing.activated['int'].disconnect()
+            existing.activated["int"].disconnect()
             # will get overwritten by the assignment below anyways
 
         notifier = QtCore.QSocketNotifier(fd, QtCore.QSocketNotifier.Read)
         notifier.setEnabled(True)
-        self._logger.debug('Adding reader callback for file descriptor {}'.format(fd))
-        notifier.activated['int'].connect(
+        self._logger.debug("Adding reader callback for file descriptor {}".format(fd))
+        notifier.activated["int"].connect(
             lambda: self.__on_notifier_ready(
-                self._read_notifiers, notifier, fd, callback, args)  # noqa: C812
+                self._read_notifiers, notifier, fd, callback, args
+            )  # noqa: C812
         )
         self._read_notifiers[fd] = notifier
 
@@ -411,7 +467,7 @@ class _QEventLoop:
         if self.is_closed():
             return
 
-        self._logger.debug('Removing reader callback for file descriptor {}'.format(fd))
+        self._logger.debug("Removing reader callback for file descriptor {}".format(fd))
         try:
             notifier = self._read_notifiers.pop(fd)
         except KeyError:
@@ -430,15 +486,16 @@ class _QEventLoop:
         else:
             # this is necessary to avoid race condition-like issues
             existing.setEnabled(False)
-            existing.activated['int'].disconnect()
+            existing.activated["int"].disconnect()
             # will get overwritten by the assignment below anyways
 
         notifier = QtCore.QSocketNotifier(fd, QtCore.QSocketNotifier.Write)
         notifier.setEnabled(True)
-        self._logger.debug('Adding writer callback for file descriptor {}'.format(fd))
-        notifier.activated['int'].connect(
+        self._logger.debug("Adding writer callback for file descriptor {}".format(fd))
+        notifier.activated["int"].connect(
             lambda: self.__on_notifier_ready(
-                self._write_notifiers, notifier, fd, callback, args)  # noqa: C812
+                self._write_notifiers, notifier, fd, callback, args
+            )  # noqa: C812
         )
         self._write_notifiers[fd] = notifier
 
@@ -447,7 +504,7 @@ class _QEventLoop:
         if self.is_closed():
             return
 
-        self._logger.debug('Removing writer callback for file descriptor {}'.format(fd))
+        self._logger.debug("Removing writer callback for file descriptor {}".format(fd))
         try:
             notifier = self._write_notifiers.pop(fd)
         except KeyError:
@@ -470,13 +527,14 @@ class _QEventLoop:
             if notifiers.get(fd, None) is notifier:
                 notifier.setEnabled(True)
             else:
-                notifier.activated['int'].disconnect()
+                notifier.activated["int"].disconnect()
 
     def __on_notifier_ready(self, notifiers, notifier, fd, callback, args):
         if fd not in notifiers:
             self._logger.warning(
-                'Socket notifier for fd {} is ready, even though it should be disabled, not calling {} and disabling'
-                    .format(fd, callback),
+                "Socket notifier for fd {} is ready, even though it should be disabled, not calling {} and disabling".format(
+                    fd, callback
+                ),
             )
             notifier.setEnabled(False)
             return
@@ -484,11 +542,11 @@ class _QEventLoop:
         # It can be necessary to disable QSocketNotifier when e.g. checking
         # ZeroMQ sockets for events
         assert notifier.isEnabled()
-        self._logger.debug('Socket notifier for fd {} is ready'.format(fd))
+        self._logger.debug("Socket notifier for fd {} is ready".format(fd))
         notifier.setEnabled(False)
         self.call_soon(
-            self.__notifier_cb_wrapper,
-            notifiers, notifier, fd, callback, args)
+            self.__notifier_cb_wrapper, notifiers, notifier, fd, callback, args
+        )
 
     # Methods for interacting with threads.
 
@@ -502,7 +560,9 @@ class _QEventLoop:
         If no executor is provided, the default executor will be used, which defers execution to
         a background thread.
         """
-        self._logger.debug('Running callback {} with args {} in executor'.format(callback, args))
+        self._logger.debug(
+            "Running callback {} with args {} in executor".format(callback, args)
+        )
         if isinstance(callback, asyncio.Handle):
             assert not args
             assert not isinstance(callback, asyncio.TimerHandle)
@@ -513,11 +573,11 @@ class _QEventLoop:
             callback, args = callback.callback, callback.args
 
         if executor is None:
-            self._logger.debug('Using default executor')
+            self._logger.debug("Using default executor")
             executor = self.__default_executor
 
         if executor is None:
-            self._logger.debug('Creating default executor')
+            self._logger.debug("Creating default executor")
             executor = self.__default_executor = QThreadExecutor()
 
         return asyncio.wrap_future(executor.submit(callback, *args))
@@ -542,23 +602,23 @@ class _QEventLoop:
         context parameter has the same meaning as in
         `call_exception_handler()`.
         """
-        self._logger.debug('Default exception handler executing')
-        message = context.get('message')
+        self._logger.debug("Default exception handler executing")
+        message = context.get("message")
         if not message:
-            message = 'Unhandled exception in event loop'
+            message = "Unhandled exception in event loop"
 
         try:
-            exception = context['exception']
+            exception = context["exception"]
         except KeyError:
             exc_info = False
         else:
             exc_info = (type(exception), exception, exception.__traceback__)
 
         log_lines = [message]
-        for key in [k for k in sorted(context) if k not in {'message', 'exception'}]:
-            log_lines.append('{}: {!r}'.format(key, context[key]))
+        for key in [k for k in sorted(context) if k not in {"message", "exception"}]:
+            log_lines.append("{}: {!r}".format(key, context[key]))
 
-        self.__log_error('\n'.join(log_lines), exc_info=exc_info)
+        self.__log_error("\n".join(log_lines), exc_info=exc_info)
 
     def call_exception_handler(self, context):
         if self.__exception_handler is None:
@@ -568,7 +628,9 @@ class _QEventLoop:
                 # Second protection layer for unexpected errors
                 # in the default implementation, as well as for subclassed
                 # event loops with overloaded "default_exception_handler".
-                self.__log_error('Exception in default exception handler', exc_info=True)
+                self.__log_error(
+                    "Exception in default exception handler", exc_info=True
+                )
 
             return
 
@@ -578,17 +640,21 @@ class _QEventLoop:
             # Exception in the user set custom exception handler.
             try:
                 # Let's try the default handler.
-                self.default_exception_handler({
-                    'message': 'Unhandled error in custom exception handler',
-                    'exception': exc,
-                    'context': context,
-                })
+                self.default_exception_handler(
+                    {
+                        "message": "Unhandled error in custom exception handler",
+                        "exception": exc,
+                        "context": context,
+                    }
+                )
             except Exception:
                 # Guard 'default_exception_handler' in case it's
                 # overloaded.
                 self.__log_error(
-                    'Exception in default exception handler while handling an unexpected error '
-                    'in custom exception handler', exc_info=True)
+                    "Exception in default exception handler while handling an unexpected error "
+                    "in custom exception handler",
+                    exc_info=True,
+                )
 
     # Debug flag management.
 
@@ -612,17 +678,17 @@ class _QEventLoop:
         try:
             cls._logger.error(*args, **kwds)
         except:  # noqa E722
-            sys.stderr.write('{!r}, {!r}\n'.format(args, kwds))
+            sys.stderr.write("{!r}, {!r}\n".format(args, kwds))
 
 
 from ._unix import _SelectorEventLoop  # noqa
 
-QSelectorEventLoop = type('QSelectorEventLoop', (_QEventLoop, _SelectorEventLoop), {})
+QSelectorEventLoop = type("QSelectorEventLoop", (_QEventLoop, _SelectorEventLoop), {})
 
-if os.name == 'nt':
+if os.name == "nt":
     from ._windows import _ProactorEventLoop
 
-    QIOCPEventLoop = type('QIOCPEventLoop', (_QEventLoop, _ProactorEventLoop), {})
+    QIOCPEventLoop = type("QIOCPEventLoop", (_QEventLoop, _ProactorEventLoop), {})
     QEventLoop = QIOCPEventLoop
 else:
     QEventLoop = QSelectorEventLoop
@@ -657,6 +723,7 @@ def asyncSlot(*args):
         @functools.wraps(fn)
         def wrapper(*args, **kwargs):
             return asyncio.ensure_future(fn(*args, **kwargs))
+
         return wrapper
 
     return outer_decorator
